<?php
// $Id: xc_browse_element.inc,v 1.1.2.3 2009/12/18 16:56:55 pkiraly Exp $

/**
 * @file
 * xc_browse_element table: el_id, tab_id, type, facet_name, search_field,
 * query_type, ui_effect, weight
 */

function xc_browse_element_get_by_tab($tab_id) {
  static $cache;
  if (!isset($cache)) {
    $cache = array();
  }
  if (!isset($cache[$tab_id])) {
    $sql = "SELECT * FROM {xc_browse_element} WHERE tab_id = %d ORDER BY weight, el_id";
    $result = db_query(db_rewrite_sql($sql, '{xc_browse_element}', 'el_id'), 
      $tab_id);
    $val = array();
    while ($data = db_fetch_object($result)) {
      $val[] = $data;
    }
    $cache[$tab_id] = $val;
  }
  return $cache[$tab_id];
}

function xc_browse_element_load($el_id) {
  $sql = "SELECT * FROM {xc_browse_element} WHERE el_id = %d";
  $result = db_query(db_rewrite_sql($sql, '{xc_browse_element}', 'el_id'), $el_id);
  $data = db_fetch_object($result);
  return $data;
} 

function xc_browse_element_title($record) {
  return t('Browse UI definition');
}

function xc_browse_element_fieldset() {
  return array(
    'el_id'        => t('Form element identifier'),
    'tab_id'       => t('Parent content pane identifier'),
    'label'        => t('A label which displaying before the form element.'),
    'type'         => t('Type of form element'), 
    'facet_name'   => t('The facet which populates the element'),
    'search_field' => t('The Solr field to search for'),
    'query_type'   => t('A Solr query type'),
    'ui_effect'    => t('JavaScript effect to apply on this element'),
    'weight'       => t('Weight'),
  );
}

function xc_browse_element_options() {
  return array(
    'type' => array( // , , , 
      'textfield' => t('text field'),
      'select' => t('dropdown select list'),
      'radios' => t('radio buttons'),
      'checkboxes' => t('check boxes'),
    ),
    'query_type' => array(
      'keyword' => t('keyword'),
      'phrase' => t('phrase'),
      'proximity' => t('proximity'),
      'truncated' => t('truncated'),
    ),
    'ui_effect' => array(),
  );
}

function xc_browse_element_view($record) {
  $output = xc_util_view($record, 
    xc_browse_element_fieldset(),
    xc_browse_element_options()
  );
  
  
  return $output;
}

function xc_browse_element_list($ui, $tab) {
  $headers = array(t('Label'), t('Name'));
  $rows = array();
  $options = xc_browse_element_options();
  $sql = "SELECT * FROM {xc_browse_element}";
  $result = db_query(db_rewrite_sql($sql, '{xc_browse_element}', 'el_id'));
  while ($data = db_fetch_object($result)) {
    $rows[] = array(
      $data->label,
      $data->name,
      l(t('view'), 'xc/browse/' . $data->el_id),
      l(t('edit'), 'xc/browse/' . $data->el_id . '/edit'),
    );
  }
  return theme('table', $headers, $rows);
}

function xc_browse_element_add($ui, $tab) {
  drupal_set_title(t('Create a new browse UI'));
  return drupal_get_form('xc_browse_element_form', $ui, $tab);
}

function xc_browse_element_form($form, $ui, $tab) {
  $schema = drupal_get_schema_unprocessed('xc_browse', 'xc_browse_element');
  drupal_set_title($schema['description']);
  $options = array(
    'omit' => array('el_id'),
    'hidden' => array(
      'tab_id' => $tab->tab_id,
      'ui_id'  => $ui->bid,
    ),
    'label' => xc_browse_element_fieldset(),
    'select' => xc_browse_element_options(),
  );
  $form = xc_util_build_autoform($schema, $options);
  //$form['ui_id']['#default_value'] = ;
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

function xc_browse_element_edit_form($form_state, $ui, $tab, $record) {
  $form = xc_browse_element_form($form_state, $ui, $tab, $record);
  $form['el_id'] = array(
    '#type'  => 'hidden',
    '#value' => $record->el_id,
  );
  
  foreach (xc_browse_element_fieldset() as $name => $label) {
    $form[$name]['#default_value'] = $record->$name;
  }
    
  $form['submit']['#value'] = t('Save');
  $form['delete'] = array(
    '#type'  => 'submit',
    '#value' => t('Delete'),
  );

  return $form;
}

function xc_browse_element_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $record = new stdClass();
  foreach (xc_browse_element_fieldset() as $name => $label) {
    $record->$name = $values[$name];
  }

  $ret_val = drupal_write_record('xc_browse_element', $record);
  if ($ret_val == SAVED_NEW) {
    drupal_set_message(t('Successfully added "%label" browse UI!',
      array('%label' => $record->label)));
    $form_state['redirect'] = 'xc/browse/' . $values['ui_id'] . '/view';
  }
  else {
    drupal_set_message(t('Unexpected error. Failed to create new browse UI.'));
  }
}

function xc_browse_element_edit_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $record = new stdClass();
  $record->el_id             = $values['el_id'];
  foreach (xc_browse_element_fieldset() as $name => $label) {
    $record->$name = $values[$name];
  }
    
  if ($form_state['clicked_button']['#value'] == t('Delete')) {
    $sql = "DELETE FROM {xc_browse_element} WHERE el_id = %d";
    $result = db_query(db_rewrite_sql($sql, '{xc_browse_element}',
      'el_id'), $record->el_id);

    if ($result == SAVED_DELETED) { // element is deleted
      drupal_set_message(t('%label removed', array('%label' => $record->label)));
      $form_state['redirect'] = 'xc/browse/' . $values['ui_id'];
    }
    else {
      drupal_set_message(t('Unexpected error. Failed to remove display template.'));
    }
  }
  else {
    $ret_val = drupal_write_record('xc_browse_element', $record, 'el_id');
    if ($ret_val == SAVED_UPDATED) {
      drupal_set_message(t('Successfully updated "%label" browse UI!',
        array('%label' => $record->label)));
      $form_state['redirect'] = 'xc/browse/' . $values['ui_id'] . '/view';
    }
    else {
      drupal_set_message(t('Unexpected error. Failed modify browse UI.'));
    }
  }
}

/**
 * Default values for XC browse element table
 * @return (String) The default values in CSV format to be insert into the table.
 */
function xc_browse_element_default() {
  $raw_values = <<<END
tab_id;label;type;facet_name;search_field;query_type;ui_effect;weight
1;Keyword(s);textfield;;;keyword;;0
1;format;radios;values:=All|videocassette=VHS ONLY|videodisc=DVD ONLY;xc:typeSMD;phrase;;2
2;Keyword(s):;select;facet:language;language;phrase;;0
2;type;radios;values:=All|videocassette=VHS ONLY|videodisc=DVD ONLY;xc:typeSMD;keyword;;0
3;Keyword(s);select;facet:creator_author;creator_author;phrase;;0
3;type;radios;values:=All|videocassette=VHS ONLY|videodisc=DVD ONLY;xc:typeSMD;phrase;;0
4;Keyword(s);select;facet:genre;genre;phrase;;0
4;Format;radios;values:=All|videocassette=VHS ONLY|videodisc=DVD ONLY;xc:typeSMD;keyword;;0
END;

  return $raw_values;
}
