<?php
// $Id: xc_browse_list.inc,v 1.1.2.3 2009/12/18 16:56:55 pkiraly Exp $

/**
 * @file
 * xc_browse_list table: lid, tab_id, label, type, field, query_type, weight
 */

function xc_browse_list_get_by_tab($tab_id) {
  static $cache;
  if (!isset($cache)) {
    $cache = array();
  }
  if (!isset($cache[$tab_id]))  {
    $sql = "SELECT * FROM {xc_browse_list} WHERE tab_id = %d ORDER BY weight, lid";
    $result = db_query(db_rewrite_sql($sql, '{xc_browse_list}', 'lid'), $tab_id);
    $val = array();
    while ($data = db_fetch_object($result)) {
      $val[] = $data;
    }
    $cache[$tab_id] = $val;
  }
  return $cache[$tab_id];
}

function xc_browse_list_load($lid) {
  $sql = "SELECT * FROM {xc_browse_list} WHERE lid = %d";
  $result = db_query(db_rewrite_sql($sql, '{xc_browse_list}', 'lid'), $lid);
  $data = db_fetch_object($result);
  return $data;
} 

function xc_browse_list_title($record) {
  return t('Browse UI definition');
}

function xc_browse_list_fieldset() {
  return array(
    'tab_id' => t('The parent content pane identifier'),
    'label' => t('The label above the list'), 
    'type' => t('The type of the navigation list'), 
    'field' => t('The Solr field to search for when the user clicks on an item'), 
    'query_type'  => t('A Solr query type'),
    'weight'  => t('Weight'),
  );
}

function xc_browse_list_options() {
  return array(
    'type' => array( 
      'ABC' => t('ABC'),
      'dates' => t('dates'),
      'facet' => t('facet'),
    ),
    'query_type' => array( //, , 
      'keyword' => t('keyword'),
      'phrase' => t('phrase'),
      'proximity' => t('proximity'),
      'truncated' => t('truncated'),
    )
  );
}

function xc_browse_list_view($record) {
  $output = xc_util_view($record, 
    xc_browse_list_fieldset(),
    xc_browse_list_options()
  );
  return $output;
}

function xc_browse_list_list() {
  $headers = array(t('Label'), t('Name'));
  $rows = array();
  $options = xc_browse_list_options();
  $sql = "SELECT * FROM {xc_browse_list}";
  $result = db_query(db_rewrite_sql($sql, '{xc_browse_list}', 'lid'));
  while ($data = db_fetch_object($result)) {
    $rows[] = array(
      $data->label,
      $data->name,
      l(t('view'), 'xc/browse/' . $data->lid),
      l(t('edit'), 'xc/browse/' . $data->lid . '/edit'),
    );
  }
  
  return theme('table', $headers, $rows);
}

function xc_browse_list_add($ui, $tab) {
  drupal_set_title(t('Create a new browse UI'));
  return drupal_get_form('xc_browse_list_form', $ui, $tab);
}

function xc_browse_list_form($form, $ui, $tab) {
  $schema = drupal_get_schema_unprocessed('xc_browse', 'xc_browse_list');
  drupal_set_title($schema['description']);
  $options = array(
    'omit' => array('lid'),
    'hidden' => array(
      'tab_id' => $tab->tab_id,
      'ui_id'  => $ui->bid,
    ),
    'label' => xc_browse_list_fieldset(),
    'select' => xc_browse_list_options(),
  );
  
  $form = xc_util_build_autoform($schema, $options);
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

function xc_browse_list_edit_form($form_state, $ui, $tab, $record) {
  $form = xc_browse_list_form($form_state, $ui, $tab, $record);
  $form['lid'] = array(
    '#type'  => 'hidden',
    '#value' => $record->lid,
  );
  
  foreach (xc_browse_list_fieldset() as $name => $label) {
    $form[$name]['#default_value'] = $record->$name;
  }
    
  $form['submit']['#value'] = t('Save');
  $form['delete'] = array(
    '#type'  => 'submit',
    '#value' => t('Delete'),
  );

  return $form;
}

function xc_browse_list_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $record = new stdClass();
  foreach (xc_browse_list_fieldset() as $name => $label) {
    $record->$name = $values[$name];
  }

  $ret_val = drupal_write_record('xc_browse_list', $record);
  if ($ret_val == SAVED_NEW) {
    drupal_set_message(t('Successfully added "%label" browse UI!',
      array('%label' => $record->label)));
    $form_state['redirect'] = 'xc/browse/' . $values['ui_id'] . '/view';
  }
  else {
    drupal_set_message(t('Unexpected error. Failed to create new browse UI.'));
  }
}

function xc_browse_list_edit_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $record = new stdClass();
  $record->lid             = $values['lid'];
  foreach (xc_browse_list_fieldset() as $name => $label) {
    $record->$name = $values[$name];
  }
    
  if ($form_state['clicked_button']['#value'] == t('Delete')) {
    $sql = "DELETE FROM {xc_browse_list} WHERE lid = %d";
    $result = db_query(db_rewrite_sql($sql, '{xc_browse_list}',
      'lid'), $record->lid);

    if ($result == SAVED_DELETED) { // element is deleted
      drupal_set_message(t('%label removed', array('%label' => $record->label)));
      $form_state['redirect'] = 'xc/browse/' . $values['ui_id'];
    }
    else {
      drupal_set_message(t('Unexpected error. Failed to remove display template.'));
    }
  }
  else {
    $ret_val = drupal_write_record('xc_browse_list', $record, 'lid');
    if ($ret_val == SAVED_UPDATED) {
      drupal_set_message(t('Successfully updated "%label" browse UI!',
        array('%label' => $record->label)));
      $form_state['redirect'] = 'xc/browse/' . $values['ui_id'] . '/view';
    }
    else {
      drupal_set_message(t('Unexpected error. Failed modify browse UI.'));
    }
  }
}

function xc_browse_list_get_ABC() {
  static $cache;
  if (empty($cache)) {
    $cache = array_merge(range('a', 'z'), range('0', '9'));
  }
  return $cache;
}

/**
 * Default values for XC browse list table
 * @return (String) The default values in CSV format to be insert into the table.
 */
function xc_browse_list_default() {
  $raw_values = <<<END
tab_id;label;type;field;query_type;weight
1;Browse by Title;ABC;dcterms:title;truncated;0
END;

  return $raw_values;
}
