<?php
// $Id: xc_metadata.source.inc,v 1.1.2.1 2009/12/16 15:00:37 mtwesley Exp $

/**
 * Get source 
 *
 * @param $source_id
 *    Source identifier
 * @param $update
 *    TRUE to refresh source cache
 * @return
 *    Source object
 */
function xc_source_get($source_id, $update = FALSE) {
  static $sources;
  if (!is_object($sources[$source_id]) || $update) {
    $sql = "SELECT source_id, name, description, type
            FROM {xc_source}
            WHERE source_id = '%d'";
    $result = db_query(db_rewrite_sql($sql, "{xc_source}", 'source_id'), $source_id);
    $source = db_fetch_object($result);
    $sources[$source_id] = $source;
  }
  return $sources[$source_id];
}

/**
 * Load source object
 *
 * @see xc_source_get()
 * @param $source_id
 *    Source identifier
 * @return
 *    Source object
 */
function xc_source_load($source_id) {
  return xc_source_get($source_id);
}

/**
 * Get default storage location for a source
 *
 * @param $source_id
 *    Source identifier
 * @return
 *    Storage location object
 */
function xc_source_get_default_location($source_id) {
  static $cache;
  if (empty($cache[$source_id])) {
    $sql = "SELECT location_id
            FROM {xc_source_locations}
            WHERE source_id = '%d'
            ORDER BY weight ASC";
    $cache[$source_id] = db_result(db_query_range(db_rewrite_sql($sql, "{xc_source_locations}", ''), $source_id, 0, 1));
  }
  $location_id = $cache[$source_id];
  return xc_location_get($location_id);
}

/**
 * Get all storage locations for a source
 *
 * @param $source_id
 *    Source identifier
 * @return
 *    An array of storage location objects for a source
 */
function xc_source_get_locations($source_id) {
  static $cache;
  if (empty($cache[$source_id])) {
    $locations = array();
    $sql = "SELECT location_id, weight
            FROM {xc_source_locations}
            WHERE source_id = '%d'
            ORDER BY weight ASC";
    $result = db_query(db_rewrite_sql($sql, "{xc_source_locations}", ''), $source_id);
    while ($_location = db_fetch_object($result)) {
      $locations[$_location->location_id] = xc_location_get($_location->location_id);
      if (is_object($locations[$_location->location_id])) {
        $locations[$_location->location_id]->weight = $_location->weight;
      }
    }
    $cache[$source_id] = $locations;
  }
  return $cache[$source_id];
}

/**
 * Set a source object to the database
 *
 * @param $source
 *    Source object
 * @return
 *    New source identifier
 */
function xc_source_set(&$source) {
  $sql = "SELECT source_id
          FROM {xc_source}
          WHERE source_id = '%d'";
  $source_id = db_result(db_query(db_rewrite_sql($sql, "{xc_source}", 'source_id'), $source->source_id));
  if ($source_id) {
    $sql2 = "UPDATE {xc_source}
             SET name = '%s', description = '%s', type = '%s'
             WHERE source_id = '%d'";
    $result = db_query(db_rewrite_sql($sql2, "{xc_source}", 'source_id'),
                       $source->name, $source->description, $source->type, $source_id);
  }
  else {
    $sql2 = "INSERT INTO {xc_source} (name, description, type)
             VALUES ('%s', '%s', '%s')";
    $result = db_query(db_rewrite_sql($sql2, "{xc_source}", 'source_id'),
                       $source->name, $source->description, $source->type);
    $source->source_id = db_last_insert_id("{xc_source}", 'source_id');
  }
  if (!$result) {
    drupal_set_message(t('Unexpected error. Unable to save metadata source'), 'error');
  }
  // Also set locations
  $source->source_id = empty($source->source_id)
    ? db_last_insert_id("{xc_source}", 'source_id')
    : $source->source_id;
  xc_source_set_locations($source);
  return $source->source_id;
}

/**
 * Set a source's locations to the database
 *
 * @param $source
 *    Source object
 */
function xc_source_set_locations($source) {
  if (is_numeric($source->source_id)) {
    $sql = "DELETE FROM {xc_source_locations}
            WHERE source_id = '%d'";
    db_query(db_rewrite_sql($sql, "{xc_source_locations}", ''), $source->source_id);

    if (is_array($source->locations)) {
      foreach ($source->locations as $location_id) {
        if (is_numeric($location_id)) {
          $sql2 = "INSERT INTO {xc_source_locations} (source_id, location_id)
                   VALUES ('%d', '%d')";
          $result = db_query(db_rewrite_sql($sql2, "{xc_source_locations}", ''), $source->source_id, $location_id);
          if (!$result) {
            drupal_set_message(t('Unexpected error. Unable to save metadata source with its locations'), 'error');
          }
        }
      }
    }
  }
}

/**
 * Unset a source from the database
 *
 * @param $source
 *    Source object
 */
function xc_source_unset($source) {
  $sql = "DELETE FROM {xc_source}
          WHERE source_id = '%d'";
  $result = db_query(db_rewrite_sql($sql, "{xc_source}", 'source_id'), $source->source_id);
  if (!$result) {
    drupal_set_message(t('Unexpected error. Unable to remove metadata source'), 'error');
  }
}

