<?php
// $Id: XCEntity.php,v 1.2.2.8 2009/11/25 12:26:10 mtwesley Exp $

class XCEntity {
  
  /** The type of entity */
  public $metadata_type;
  
  /** The ID of entity */
  public $metadata_id;

  /** A unique identifier **/
  public $identifier;
  
  /** The Drupal node type */
  public $node_type = NULL;
  
  /** The Drupal node ID */
  public $node_id = NULL;
  
  /** The content of the object as array */
  public $metadata = array();
  
  /** The source data in its original format */
  public $raw;
  
  /** The name of the format of the original data */
  public $format;
  
  /** The timestap */
  public $timestamp;
  
  /** The relationships of the entity */
  public $relationships = array();
  
  /** Has the entity been built? */
  public $built = FALSE;
  
  /** Has the entity been stored? */
  public $stored = FALSE;

  /** Locks on the metadata **/
  public $locks = array();

  /** Source of metadata **/
  public $source_id = NULL;

  /** Location of metadata **/
  public $locations = array();

  /** Additional properties **/
  public $properties = array();

  /**
   * Creates a new entity
   * @param $type The type of entity
   * @param $identifier   The id of entity
   * @return XCEntity The newly created entity
   */
  public function __construct($arg = NULL) {
    if (is_array($arg) || is_object($arg)) {
      $arg = get_object_vars($arg);
      
      if (!empty($arg['metadata_id'])) {
        $metadata_id = $arg['metadata_id'];
      }
      if (empty($metadata_id) && !empty($arg['node_id'])) {
        $metadata_id = _node_id_to_metadata_id($arg['node_id']);
      }
      if (empty($metadata_id) && !empty($arg['nid'])) {
        $metadata_id = _node_id_to_metadata_id($arg['nid']);
      }
      if (empty($metadata_id) && !empty($arg['identifier'])) {
        $metadata_id = _identifier_to_metadata_id($arg['identifier']);
      }
      foreach ($arg as $property => $value) {
        $this->$property = $value;
      }
    }
    else {
      $metadata_id = $arg;
    }

    if (empty($metadata_id)) {
      return;
    }

    $this->metadata_id = $metadata_id;

    $_cached = xc_cache_get('metadata', _object_to_qualified($this, 'metadata'));
    if (!empty($_cached)) {
      $this->metadata_type = $_cached->metadata_type;
      $this->node_type     = $_cached->node_type;
      $this->node_id       = $_cached->node_id;
      $this->metadata      = $_cached->metadata;
      $this->raw           = $_cached->raw;
      $this->format        = $_cached->format;
      $this->timestamp     = $_cached->timestamp;
      $this->relationships = $_cached->relationships;
      $this->built         = $_cached->built;
      $this->stored        = $_cached->stored;
      $this->locks         = $_cached->locks;
      $this->source_id     = $_cached->source_id;
      $this->locations     = $_cached->locations;
      $this->properties    = $_cached->properties;
    }
    else {
      _xc_retrieve($this);
    }
  }

  public static function load($metadata_id) {
    return new XCEntity(array('metadata_id' => $metadata_id));
  }

  public static function loadNodeID($node_id) {
    return new XCEntity(array('node_id' => $node_id));
  }
  
  public function get_properties() { 
    return xc_entity_load_properties($this);
  }

  public function get_relationships() { 
    return xc_entity_load_relationships($this);
  }

  public function get_source() {
    xc_source_get($this->source_id);
  }

  public function get_default_location() {
    xc_source_get_default_location($this->source_id);
  }

  public function get_all_locations() {
    xc_source_get_locations($this->source_id);
  }

  public function get_definition() { 
    return xc_metadata_entity_get($this->metadata_type);
  }

  public function get_node() {
    return node_load(array('nid' => $this->node_id, 'type' => $this->node_type));
  }

  public function set_relationships() { 
    $object = xc_entity_set_relationships($this);
    if ($object) {
      return $this;
    }
  }

  /**
   * Call _xc_build, which constructs metadata objects from its raw input data
   * based on the raw import format.
   */
  public function build($params = array()) { 
    _xc_build($this, $this->format, $params);
  }

  /**
   * Call _xc_transform
   * @see _xc_transform
   * @param $format
   * @return unknown_type
   */
  public function transform($format = '', $params = array()) {
    return _xc_transform($this, $format, $params);
  }

  public function store($locations = array(), $params = array()) {
    _xc_store($this, $locations, $params);
  }

  public function retrieve($locations = array(), $params = array()) {
    _xc_retrieve($this, $locations, $params);
  }

  public function remove($locations = array(), $params = array()) {
    _xc_remove($this, $locations, $params);
  }

  public function alter($params = array()) {
    _xc_alter($this, $params);
  }

  public function validate() { }

  public function clean() { }

  public function fetch_field($field, $delta = 0) {
    return xc_fetch_field($this, $field, $delta);
  }

  public function fetch_attribute($field, $attribute, $delta = 0) {
    return xc_fetch_field($this, $field, $attribute, $delta);
  }

  public function alter_field($field, $delta = 0, $alter = NULL, $value = NULL) {
    xc_alter_field($this, $field, $delta, $alter, $value);
  }

  public function alter_attribute($field, $attribute, $delta = 0, $alter = NULL, $value = NULL) {
    xc_alter_field($this, $field, $attribute, $delta, $alter, $value);
  }

  public function is_locked() {
    // TODO: Do a little more than just say if its locked
  }

}
