<?php

function xc_search_mlt_title($set) {
  return $set->name;
}

function xc_search_mlt_load($mlt_id) {
  $sql = "SELECT * FROM {xc_search_mlt} WHERE mlt_id = %d";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_mlt}', 'mlt_id'), $mlt_id);
  $data = db_fetch_object($result);
  return $data;
} 

function xc_search_mlt_fieldset() {
  return array(
    'name' => t('Name'),
    'fl' => t('Fields'),
    'mintf' => t('Minimum Term Frequency'),
    'mindf' => t('Minimum Document Frequency'),
    'minwl' => t('Minimum word length'),
    'maxwl' => t('Maximum word length'),
    'maxqt' => t('Maximum query terms'),
    'maxntp' =>  t('Maximum number of tokens to parse'),
    'boost' => t('Boosting'),
    'qf' => t('Boosting factors'),
    'count' => t('Limit'),
    'match_include' => t('Include matched document?'),
    'match_offset' => t('Similarity of which result?'),
    'interestingTerms' => t('Show the "interesting" terms'),
  );
}

function xc_search_mlt_options() {
  global $XC_SEARCH_GLOBAL_OPTIONS;
  return array(
    'boost' => $XC_SEARCH_GLOBAL_OPTIONS['true_false_solr'],
    'match_include' => $XC_SEARCH_GLOBAL_OPTIONS['true_false_solr'],
    'interestingTerms'  => array(
      'list'    => t('list'),
      'details' => t('details'),
      'none'    => t('none'),
    ),
  );
}

function xc_search_mlt_view($record) {
  return xc_util_view($record, 
    xc_search_mlt_fieldset(),
    xc_search_mlt_options()
  );
}

function xc_search_mlt_list() {
  
  $headers = array(t('Name'), t('Similar fields'));
  $rows = array();
  
  $sql = "SELECT mlt_id, name, fl FROM {xc_search_mlt}";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_mlt}', 'mlt_id'));
  while ($data = db_fetch_object($result)) {
    $rows[] = array(
      l($data->name, 'xc/search/displaying/mlt/'.$data->mlt_id), 
      $data->fl,
      l(t('edit'), 'xc/search/displaying/mlt/'.$data->mlt_id . '/edit'),
      l(t('delete'), 'xc/search/displaying/mlt/'.$data->mlt_id . '/delete'),
    );
  }
  return theme('table', $headers, $rows);
}

/**
 * Add a new server
 * @return string The add server page with a form
 */
function xc_search_mlt_add() {
  return drupal_get_form('xc_search_mlt_form');
}

function xc_search_mlt_form() {
  $schema = drupal_get_schema_unprocessed('xc_search', 'xc_search_mlt');
  drupal_set_title($schema['description']);
  $options = array(
    'omit' => array('mlt_id'),
    'select' => xc_search_mlt_options(),
    'label' => xc_search_mlt_fieldset(),
  );
  $form = xc_util_build_autoform($schema, $options);
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );

  return $form;
}

function xc_search_mlt_edit_form($form_state, $set) {
  $form = xc_search_mlt_form();
  $form['mlt_id'] = array(
    '#type'  => 'hidden',
    '#value' => $set->mlt_id,
  );
  foreach (xc_search_mlt_fieldset() as $name => $label) {
    $form[$name]['#default_value'] = $set->$name;
  }

  $form['submit']['#value'] = t('Save');
  $form['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete'),
  );

  return $form;
} 

function xc_search_mlt_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $record = new stdClass();
  foreach (xc_search_mlt_fieldset() as $name => $label) {
    $record->$name = $values[$name];
  }
  $ret_val = drupal_write_record('xc_search_mlt', $record);
  if ($ret_val == SAVED_NEW) {
    drupal_set_message(t('Successfully added "More like this" settings "%name"',
                         array('%name' => $record->name)));
    $form_state['redirect'] = 'xc/search/displaying/mlt/' . $record->mlt_id;
  }
  else {
    drupal_set_message(t('Unexpected error. Failed to create new "More like this" settings.'));
  }
}

function xc_search_mlt_edit_form_submit($form, &$form_state) {
  $values = $form_state['clicked_button']['#post'];
  $record = new stdClass();
  $record->mlt_id      = $values['mlt_id'];
  $record->name        = $values['name'];

  if ($form_state['clicked_button']['#value'] == t('Delete')) {
    $sql = "DELETE FROM {xc_search_mlt} WHERE mlt_id = %d";
    $result = db_query(db_rewrite_sql($sql, '{xc_search_mlt}', 'mlt_id'), 
      $record->mlt_id);

    if ($result == SAVED_DELETED) { // repository is deleted
      drupal_set_message(t('%name removed', array('%name' => $record->name)));
      $form_state['redirect'] = 'xc/search/displaying/mlt/list';
    }
    else {
      drupal_set_message(t('Unexpected error. Failed to remove %name.', 
        array('%name' => $record->name)));
    }
  }
  elseif ($form_state['clicked_button']['#value'] == t('Save')) {
    foreach (xc_search_mlt_fieldset() as $name => $label) {
      $record->$name = $values[$name];
    }

    $result = drupal_write_record('xc_search_mlt', $record, 'mlt_id');
    if ($result == SAVED_UPDATED) { // repository is updated
      drupal_set_message(t('%name modified', array('%name' => $record->name)));
      $form_state['redirect'] = 'xc/search/displaying/mlt/'. $record->mlt_id . '/view';
    }
    else {
      drupal_set_message(t('Unexpected error. Failed to modify %name.', 
        array('%name' => $record->name)));
    }
  }
  else {
    drupal_set_message(t('Unexpected error. Something else happened.'));
  }
}

function xc_search_mlt_default() {
  log_info('xc search', 'mlt default');
  $raw_values = <<<END
name;fl;mintf;mindf;minwl;maxwl;maxqt;maxntp;boost;qf;count;match_include;match_offset;interestingTerms
MLT for full record display;dcterms__title_t,rdarole__author_t;1;2;0;0;0;0;true;;10;false;0;list
END;

  return $raw_values;
}

