<?php
/**
 * @file
 * All functions to create, modify and delete predefined facets (xc_search_facet table).
 * Fields are: facet_id, name, label, type (= suffix for a Solr dynamic field)
 */

function xc_search_facet_load($facet_id) {
  static $xc_search_facet_load_cache;
  if (!isset($xc_search_facet_load_cache[$facet_id])) {
    $sql = "SELECT * FROM {xc_search_facet} WHERE facet_id = '%s'";
    $result = db_query(db_rewrite_sql($sql, '{xc_search_facet}', 'facet_id'), 
      $facet_id);
    $record = db_fetch_object($result);
    $xc_search_facet_load_cache[$facet_id] = $record;
  }
  return $xc_search_facet_load_cache[$facet_id];
}

function xc_search_facet_fieldset() {
  return array(
    'name'  => t('machine name'), 
    'label' => t('display name'), 
    'type'  => t('Solr type'), 
  );
}

function xc_search_facet_options($record = NULL) {
  $fields = xc_format_get_fields();
  $fields_array = array();
  foreach ($fields as $field) {
    $fields_array[$field] = $field;
  }
  
  $options = array(
    'fields' => $fields_array,
    'type'   => xc_solr_field_type_get_types()
  );
  return $options;
}

function xc_search_facet_list() {
  $output = '<p>' . theme('image', 'http://wiki.apache.org/moin_static184/modernized/img/alert.png', 
    t('Alert!'), t('Alert!'), NULL, FALSE) . ' ' . t('Alert! Advanced settings! Modify these settings 
    very carefully, and only after you have read the manual!') . '</p>';
  $output .= '<p>' . t('The list of predefined facets. These facets should be defined before indexing.
    Each facet contain the content of one or more fields. We call the facet-field pairs to mapping. 
    You can see the mappings !grouped_by_facets or !grouped_by_fields. You can restore default values
    comes with the module with click on !restore_defaults',
    array(
      '!grouped_by_facets' => l(t('grouped by facets'), 'xc/search/indexing/facets/listfacets'),
      '!grouped_by_fields' => l(t('grouped by fields'), 'xc/search/indexing/facets/listfields'),
      '!restore_defaults'  => l(t('restore defaults'),  'xc/search/indexing/facets/restore_defaults'),
    )) . '</p>';
  
  $types = xc_solr_field_type_get_all();
  $sql = "SELECT * FROM {xc_search_facet} ORDER BY name"; 
  $result = db_query(db_rewrite_sql($sql, '{xc_search_facet}', 'facet_id'));
  while ($record = db_fetch_object($result)) {
    $rows[] = array(
      $record->label,
      $record->name,
      l($types[$record->type]->label, 'xc/solr/field_type/' . $types[$record->type]->type_id),
      l('edit',   'xc/search/indexing/facets/' . $record->facet_id . '/edit'),
      l('delete', 'xc/search/indexing/facets/' . $record->facet_id . '/delete'),
    );
  }
  $labels = xc_search_facet_fieldset();
  $headers = array($labels['label'], $labels['name'], $labels['type'], t('edit'), t('delete'));
  $output .= theme('table', $headers, $rows);
  return $output;
}

function xc_search_facet_add() {
  drupal_set_title('Create a new facet');
  return drupal_get_form('xc_search_facet_form');
}

function xc_search_facet_form(&$form_state, $record = NULL) {
  $schema = drupal_get_schema_unprocessed('xc_search', 'xc_search_facet');
  drupal_set_title($schema['description']);

  $options = array(
    'omit' => array('facet_id'),
    'hidden' => array(),
    'label' => xc_search_facet_fieldset(),
    'select' => xc_search_facet_options($record),
  );
  $form = xc_util_build_autoform($schema, $options);
  
  $form['fields'] = array(
    '#title' => t('Fields'),
    '#type' => 'select',
    '#options' => $options['select']['fields'],
    '#multiple' => TRUE,
  );
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * Creating a two column layout for the adding/modifying facet form
 * @param $form Input 'raw' form structure
 * @return The layout
 */
function theme_xc_search_facet_form($form) {
  $rows[] = array(
    array(
      'data' => drupal_render($form['name']) 
              . drupal_render($form['label']) 
              . drupal_render($form['type']), 
      'width' => '50%', 'valign' => 'top'
    ), 
    array('data' => drupal_render($form['fields']), 'valign' => 'top')
  );
  $output .= theme('table', NULL, $rows);
  $output .= drupal_render($form);
  return $output;
}

function xc_search_facet_edit_form($form_state, $record = NULL) {
  $mapping = xc_search_facet_for_edit_load($record->name);
  $fields = $mapping->field;
  $form = xc_search_facet_form($form_state, $record);
  drupal_set_title(t('Edit %facet facet', array('%facet' => $record->label)));
  $form['facet_id'] = array(
    '#type'  => 'hidden',
    '#value' => $record->facet_id,
  );
  $form['original_fields'] = array(
    '#type'  => 'hidden',
    '#value' => serialize($fields),
  );
  foreach (xc_search_facet_fieldset() as $name => $label) {
    $form[$name]['#default_value'] = $record->$name;
  }
  $form['fields']['#default_value'] = $fields;
  
  $form['submit']['#value'] = t('Modify');
  $form['delete'] = array(
    '#type'  => 'submit',
    '#value' => t('Delete'),
  );
  $form['cancel'] = array(
    '#type'  => 'submit',
    '#value' => t('Cancel'),
  );
  return $form;
}

function theme_xc_search_facet_edit_form($form) {
  return theme_xc_search_facet_form($form);
}

/**
 * Submit form
 * @param $form
 * @param $form_state
 * @return unknown_type
 */
function xc_search_facet_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $fields = $values['fields'];

  // prepare data
  $record = new stdClass();
  foreach (xc_search_facet_fieldset() as $name => $label) {
    $record->$name = $values[$name];
  }
  
  // save new facet
  $ret_val = drupal_write_record('xc_search_facet', $record);
  if ($ret_val == SAVED_NEW) {
    drupal_set_message(t('Successfully created %name facet.', array('%name' => $record->label)));

    // save new field mapping to facet
    foreach ($fields as $field) {
      // create it only if it doesn't exist
      if (xc_search_facet_mapping_exists($field, $facet)) {
        continue;
      }
      $mapping_record = new stdClass();
      $mapping_record->field = $field;
      $mapping_record->facet = $record->name;
      $ret_val = drupal_write_record('xc_search_field_to_facet', $mapping_record);
      if ($ret_val == SAVED_NEW) {
        drupal_set_message(t('Successfully added %field field for %facet facet.',
          array('%facet' => $record->label, '%field' => $mapping_record->field)));
      }
      else {
        drupal_set_message(t('Unexpected error. Failed to create new field-facet mapping.') . db_error());
      }
    }
  }
  else {
    drupal_set_message(t('Unexpected error. Failed to create new facet.'));
  }
  $form_state['redirect'] = 'xc/search/indexing/facets';
}

function xc_search_facet_edit_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $fields = $values['fields'];
  $original_fields = unserialize($values['original_fields']);
  if (!is_array($original_fields)) {
     $original_fields = array();
  }

  // prepare data
  $record = new stdClass();
  $record->facet_id = $values['facet_id'];
  foreach (xc_search_facet_fieldset() as $name => $label) {
    $record->$name = $values[$name];
  }
  $op = isset($form_state['values']['op']) ? $form_state['values']['op'] : ''; 
  // $form_state['clicked_button']['#value'] 
  
  // delete
  if ($op == t('Delete')) {
    $sql = "DELETE FROM {xc_search_facet} WHERE facet_id = %d";
    $result = db_query(db_rewrite_sql($sql, '{xc_search_facet}', 'facet_id'), 
      $record->facet_id);
    if ($result == SAVED_DELETED) { // facet is deleted
      drupal_set_message(t('Facet %name removed', array('%name' => $record->label)));
      // delete mappings
      $sql = "DELETE FROM {xc_search_field_to_facet} WHERE facet = '%s'";
      $result = db_query(db_rewrite_sql($sql, '{xc_search_field_to_facet}', 
        'ftof_id'), $record->name);
      if ($result == SAVED_DELETED) { // facet is deleted
        drupal_set_message(t('All fields for %facet facet removed', 
          array('%facet' => $record->label)));
      }
      else {
        drupal_set_message(t('Unexpected error. Failed to remove facet.'));
      }
    }
    else {
      drupal_set_message(t('Unexpected error. Failed to remove facet.'));
    }
  }
  elseif ($op == t('Modify')) {
    $ret_val = drupal_write_record('xc_search_facet', $record, 'facet_id');
    if ($ret_val == SAVED_UPDATED) {
      drupal_set_message(t('Successfully updated %name facet', array('%name' => $record->label)));
      // TODO: update fields
      $deletables = array_diff($original_fields, $fields);
      $new_fields = array_diff($fields, $original_fields);
      foreach ($deletables as $field) {
        $sql = "DELETE FROM {xc_search_field_to_facet} 
                WHERE facet = '%s'
                  AND field = '%s'";
        $result = db_query(db_rewrite_sql($sql, '{xc_search_field_to_facet}', 
          'ftof_id'), $record->name, $field);
        if ($result == SAVED_DELETED) { // facet is deleted
          drupal_set_message(t('%field->%facet removed', 
            array('%facet' => $record->label, '%field' => $field)));
        }
        else {
          drupal_set_message(t('Unexpected error. Failed to delete facet mapping.'));
        }
      }
      foreach ($new_fields as $field) {
        $mapping_record = new stdClass();
        $mapping_record->field = $field;
        $mapping_record->facet = $record->name;
        $ret_val = drupal_write_record('xc_search_field_to_facet', $mapping_record);
        if ($ret_val == SAVED_NEW) {
          drupal_set_message(t('Successfully added %field field for %facet facet.',
            array('%facet' => $record->label, '%field' => $mapping_record->field)));
        }
        else {
          drupal_set_message(t('Unexpected error. Failed to create new field-facet mapping.') . db_error());
        }
      }
    }
    else {
      drupal_set_message(t('Unexpected error. Failed to create new facet.'));
    }
  }
  $form_state['redirect'] = 'xc/search/indexing/facets';
}

function xc_search_facet_restore_defaults_form() {
  $form['submit'] = array(
    '#type' => 'submit',
    '#prefix' => t('Are you sure, that you would like to restore default values? You will lost 
    all your settings and the module\'s default values will be reset.') . '<br/>',
    '#value' => t('Yes'),
  );
  $form['cancel'] = array(
    '#type' => 'submit',
    '#value' => t('Cancel'),
  );
  return $form;
}

function xc_search_facet_restore_defaults_form_submit($form, &$form_state) {
  $op = isset($form_state['values']['op']) ? $form_state['values']['op'] : ''; 
  if ($op == t('Yes')) {
    $sql = "DELETE FROM {xc_search_facet}";
    $result = db_query(db_rewrite_sql($sql, '{xc_search_facet}', 'facet_id'));
    if ($result == SAVED_DELETED) {
      drupal_set_message(t('All facets were removed'));
      xc_util_bulk_insert('xc_search_facet', xc_search_facet_default());
      $sql = "DELETE FROM {xc_search_field_to_facet}";
      $result = db_query(db_rewrite_sql($sql, '{xc_search_field_to_facet}', 
        'ftof_id'));
      if ($result == SAVED_DELETED) {
        drupal_set_message(t('All facet mapping were removed'));
        xc_util_bulk_insert('xc_search_field_to_facet', xc_search_field_to_facet_default());
      }
      else {
        drupal_set_message(t('Unexpected error. Failed to remove facet mappings.'));
      }
    }
    else {
      drupal_set_message(t('Unexpected error. Failed to remove facet.'));
    }
  }
  $form_state['redirect'] = 'xc/search/indexing/facets';
}

function xc_search_facet_default() {
  $raw_values = <<<END
name;label;type
authors_contributors;Authors/Contributors;_fc
creator_author;Creators/Authors;_fc
other_contributors;Other contributors;_fc
format;Format;_fc
language;Language;_fc
date;Date;_df
subject;Subject;_fc
topic;Topic;_fc
classification;Classification;_fc
region;Region;_fc
genre;Genre;_fc
related_resources;Related resources;_fc
type_of_content;Type of content;_fc
END;

  return $raw_values;
}

function xc_search_facet_exists($facet) {
  $sql = "SELECT COUNT(*) FROM {xc_search_facet} WHERE name = '%s'";
  $count = db_result(db_query(db_rewrite_sql($sql, '{xc_search_facet}', 
    'facet_id'), $facet));
  return ($count == 0 ? FALSE : TRUE);
}

