<?php
/**
 * @file
 * Functions for manipulating xc_search_field_type table.
 * Fields are: field_type_id, metadata_schema, field, type (= the suffix for that Solr field)
 */

function xc_search_field_type_metadata_load($metadata_schema) {
  if (!isset($metadata_schema) || trim($metadata_schema) == "") {
    return NULL;
  }
  $fields = xc_format_get_fields($metadata_schema);
  return array(
    'metadata_schema' => $metadata_schema,
    'fields' => $fields
  );
}

function xc_search_field_type_view() {
  $output = t('Select a metadata schema to list its fields, and the connected Solr data types.');
  $schemas = xc_util_get_schemas();
  
  $items = array();
  foreach ($schemas as $key => $value) {
    $items[] = l($value, 'xc/search/indexing/field_type/' . $key);
  }
  $output .= theme('item_list', $items);
  return $output;
}

function xc_search_field_type_get_field_types() {
  $sql = "SELECT * FROM {xc_search_field_type}";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_field_type}', 
    'field_type_id'));
  $values = array();
  while ($data = db_fetch_object($result)) {
    $values[$data->field] = array('id' => $data->field_type_id, 'type' => $data->type);
  }
  return $values;
}

function xc_search_field_type_get_field_types_noid() {
  $sql = "SELECT * FROM {xc_search_field_type}";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_field_type}', 
    'field_type_id'));
  $values = array();
  while ($data = db_fetch_object($result)) {
    $values[$data->field][] = $data->type;
  }
  return $values;
}

function xc_search_field_type_form(&$form_state, $metadata_schema) {
  $schema_name = $metadata_schema['metadata_schema'];
  
  drupal_set_title(t('Map fields of %schema schema to Solr dynamic types', 
    array('%schema' => $schema_name)));

  $fields = $metadata_schema['fields'];
  $fields = array_merge($fields, xc_search_attribute_get_solr_fields($schema_name));

  $existing_field_types = xc_search_field_type_get_all_by_schema($schema_name);
  $types = xc_solr_field_type_get_all();
  $form['metadata_schema'] = array(
    '#type' => 'hidden',
    '#value' => $schema_name,
  );

  foreach ($fields as $field => $label) {
    $default_types = array('_t');
    if (isset($existing_field_types[$field])) {
      $default_types = $existing_field_types[$field];
    }
    foreach ($types as $suffix => $type) {
      $key = 'type---'.$field.'---' . $suffix;
      $form['types'][$field][$suffix][$key] = array(
        '#title' => $suffix,
        '#type' => 'checkbox',
        '#return_value' => $suffix, 
      );
      if (in_array($suffix, $default_types)) {
        $form['types'][$field][$suffix][$key]['#attributes']['checked'] = 'checked';
      }
    }
  }
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  $form['restore_defaults'] = array(
    '#type' => 'submit',
    '#value' => t('Restore defaults'),
  );
  $form['cancel'] = array(
    '#type' => 'submit',
    '#value' => t('Cancel'),
  );
  return $form;
}

function theme_xc_search_field_type_form($form) {
  $output = '<p>' . theme('image', 'http://wiki.apache.org/moin_static184/modernized/img/alert.png', 
    t('Alert!'), t('Alert!'), NULL, FALSE) . ' ' . t('Alert! Advanced settings! Modify these settings 
    very carefully, and only after you have read the manual!') . '</p>';
  
  $output .= t('There are lots of different dynamic Solr field types. If you want to add more, or 
    remove uneccessary ones, first you should change the Solr\'s schema.xml then register the
    changes in !solr_field_type form, because the Toolkit can not read them out directly from Solr. 
    This table always shows the actually registered Solr field types. 
    Here you can map one or more Solr field types to a schema field or attribute. 
    We set the default type to text, which tokenizes and store the input values &mdash; the best choose
    for searching for terms. 
    Those fields which still has not mapped to a type are displayed in <font color="red">red</font>.',
    array('!solr_field_type' => l('Solr field types', 'xc/solr/field_type'))
  );
  $metadata_schema = $form['metadata_schema']['#value'];
  $existing_field_types = xc_search_field_type_get_all_by_schema($metadata_schema);
  
  $rows = array();
  $row = array();
  foreach (element_children($form['types']) as $field) {
    $row = array(
      (isset($existing_field_types[$field]) 
        ? $field 
        : array('data' => $field, 'style' => 'color:red'))
    );
    foreach (element_children($form['types'][$field]) as $type) {
      $row[] = array('data' => drupal_render($form['types'][$field][$type]));
    }
    $rows[] = $row;
  }
  $header = array(t('field'));
  foreach (xc_solr_field_type_get_all() as $suffix => $type) {
    $header[] = $type->label;
  }
  
  $output .= theme('table', $header, $rows);
  $output .= drupal_render($form);
  return $output;
}

function xc_search_field_type_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $metadata_schema = $values['metadata_schema'];
  $op = isset($form_state['values']['op']) ? $form_state['values']['op'] : '';
  $solr_types = xc_solr_field_type_get_all();
  if ($op == t('Save')) { 
    $existing_field_types = xc_search_field_type_get_all_by_schema($metadata_schema);
  
    // build a same array from user data
    $actual_field_types = array();
    foreach ($values as $key => $value) {
      if ($value != '0' && preg_match('/^type---(.*?)---(.*?)$/', $key, $matches)) {
        $actual_field_types[$matches[1]][] = $value;
      }
    }
    
    // comparing the differences between stored values and actual ones
    $fields_to_delete = array_diff(array_keys($existing_field_types), array_keys($actual_field_types));
    $fields_to_create = array_diff(array_keys($actual_field_types), array_keys($existing_field_types));
    $common_fields = array_intersect(array_keys($existing_field_types), array_keys($actual_field_types));
    $types_to_delete = array();
    $types_to_create = array();
    foreach ($common_fields as $field) {
      $del_types = array_diff($existing_field_types[$field], $actual_field_types[$field]);
      if (count($del_types)) {
        $types_to_delete[$field] = $del_types;
      }
      $new_types = array_diff($actual_field_types[$field], $existing_field_types[$field]);
      if (count($new_types)) {
        $types_to_create[$field] = $new_types;
      }
    }
    foreach ($fields_to_create as $field) {
      foreach ($actual_field_types[$field] as $type) {
        $types_to_create[$field][] = $type;
      }
    }
  
    // delete old fields
    foreach ($fields_to_delete as $field) {
      $sql = "DELETE FROM {xc_search_field_type}
              WHERE metadata_schema = '%s'
                AND field = '%s'";
      $result = db_query(db_rewrite_sql($sql, '{xc_search_field_type}', 
        'field_type_id'), $metadata_schema, $field);
      if ($result == SAVED_DELETED) { // repository is deleted
        drupal_set_message(t('%field removed', array('%field' => $field)));
      }
      else {
        drupal_set_message(t('Unexpected error. Failed to remove Solr field type.'));
      }
    }

    // delete old types
    foreach ($types_to_delete as $field => $types) {
      foreach ($types as $type) {
        $sql = "DELETE FROM {xc_search_field_type}
                WHERE metadata_schema = '%s'
                  AND field = '%s'
                  AND type  = '%s'";
        $result = db_query(db_rewrite_sql($sql, '{xc_search_field_type}', 
          'field_type_id'), $metadata_schema, $field, $type);
        if ($result == SAVED_DELETED) { // repository is deleted
          drupal_set_message(t('%field->%type removed', array('%field' => $field, 
            '%type' => $solr_types[$type]->type)));
        }
        else {
          drupal_set_message(t('Unexpected error. Failed to remove Solr field type.'));
        }
      }
    }

    // create new types
    foreach ($types_to_create as $field => $types) {
      foreach ($types as $type) {
        $record = new stdClass();
        $record->metadata_schema = $metadata_schema;
        $record->field = $field;
        $record->type  = $type;
        $ret_val = drupal_write_record('xc_search_field_type', $record);
        if ($ret_val == SAVED_NEW) {
          drupal_set_message(t('Successfully registered %field field as %type type.',
            array('%field' => $record->field, '%type' => $solr_types[$type]->type)));
        } 
        else {
          drupal_set_message(t('Unexpected error. Failed to register field type.'));
        }
      }
    }
  } // save
  elseif ($op == t('Restore defaults')) {
    // TODO:
    // delete all values belonging to the actual schema
    $sql = "DELETE FROM {xc_search_field_type} WHERE metadata_schema = '%s'";
    $result = db_query(db_rewrite_sql($sql, '{xc_search_field_type}', 
      'field_type_id'), $metadata_schema);
    if ($result == SAVED_DELETED) { // types are deleted
       drupal_set_message(t('All types are removed from %schema schema', 
         array('%schema' => $metadata_schema)));
    }
    else {
      drupal_set_message(t('Unexpected error. Failed to remove Solr field type.'));
    }
    // insert values from defaults
    $records = xc_util_create_bulk_records(xc_search_field_type_default());
    foreach ($records as $record) {
      if ($record->metadata_schema == $metadata_schema) {
        $ret_val = drupal_write_record('xc_search_field_type', $record);
        if ($ret_val == SAVED_NEW) {
          drupal_set_message(t('Successfully registered %field field as %type type.',
            array('%field' => $record->field, '%type' => $solr_types[$record->type]->label)));
        } 
        else {
          drupal_set_message(t('Unexpected error. Failed to register field type.'));
        }
      }
    }
  }
  $form_state['redirect'] = 'xc/search/indexing/field_type';
}

function xc_search_field_type_default() {
  $defaults = <<<END
metadata_schema;field;type
oai_dc;dc:contributor;_t
oai_dc;dc:coverage;_t
oai_dc;dc:creator;_t
oai_dc;dc:date;_t
oai_dc;dc:description;_t
oai_dc;dc:format;_t
oai_dc;dc:identifier;_t
oai_dc;dc:language;_t
oai_dc;dc:publisher;_t
oai_dc;dc:relation;_t
oai_dc;dc:rights;_t
oai_dc;dc:source;_t
oai_dc;dc:subject;_t
oai_dc;dc:title;_t
oai_dc;dc:type;_t
oai_dc;dc:date;_d
xc;dcterms:accessRights;_t
xc;dcterms:accrualMethod;_t
xc;dcterms:accrualPeriodicity;_t
xc;dcterms:accrualPolicy;_t
xc;dcterms:alternative;_t
xc;dcterms:audience;_t
xc;dcterms:available;_t
xc;dcterms:bibliographicCitation;_t
xc;dcterms:conformsTo;_t
xc;dcterms:contributor;_t
xc;dcterms:coverage;_t
xc;dcterms:created;_t
xc;dcterms:creator;_t
xc;dcterms:date;_t
xc;dcterms:dateAccepted;_t
xc;dcterms:dateCopyrighted;_t
xc;dcterms:dateSubmitted;_t
xc;dcterms:description;_t
xc;dcterms:educationLevel;_t
xc;dcterms:extent;_t
xc;dcterms:format;_t
xc;dcterms:hasFormat;_t
xc;dcterms:hasPart;_t
xc;dcterms:hasVersion;_t
xc;dcterms:identifier;_t
xc;dcterms:instructionMethod;_t
xc;dcterms:isFormatOf;_t
xc;dcterms:isPartOf;_t
xc;dcterms:isReferencedBy;_t
xc;dcterms:isReplacedBy;_t
xc;dcterms:isRequiredBy;_t
xc;dcterms:isVersionOf;_t
xc;dcterms:issued;_t
xc;dcterms:language;_t
xc;dcterms:license;_t
xc;dcterms:mediator;_t
xc;dcterms:medium;_t
xc;dcterms:modified;_t
xc;dcterms:provenance;_t
xc;dcterms:publisher;_t
xc;dcterms:references;_t
xc;dcterms:relation;_t
xc;dcterms:replaces;_t
xc;dcterms:requires;_t
xc;dcterms:rights;_t
xc;dcterms:rightsHolder;_t
xc;dcterms:source;_t
xc;dcterms:spatial;_t
xc;dcterms:subject;_t
xc;dcterms:tableOfContents;_t
xc;dcterms:temporal;_t
xc;dcterms:title;_t
xc;dcterms:type;_t
xc;dcterms:valid;_t
xc;dcterms:version;_t
xc;rdarole:artist;_t
xc;rdarole:author;_t
xc;rdarole:compiler;_t
xc;rdarole:composer;_t
xc;rdarole:director;_t
xc;rdarole:editor;_t
xc;rdarole:illustrator;_t
xc;rdarole:performer;_t
xc;rdarole:producer;_t
xc;rdarole:speaker;_t
xc;rdarole:translator;_t
xc;rdvocab:artisticAndOrTechnicalCredits;_t
xc;rdvocab:awards;_t
xc;rdvocab:coordinatesOfCartographicContent;_t
xc;rdvocab:dimensions;_t
xc;rdvocab:dissertationOrThesesInformation;_t
xc;rdvocab:editionStatement;_t
xc;rdvocab:frequency;_t
xc;rdvocab:identifierForTheExpression;_t
xc;rdvocab:identifierForTheWork;_t
xc;rdvocab:illustration;_t
xc;rdvocab:modeOfIssuance;_t
xc;rdvocab:natureOfTheContent;_t
xc;rdvocab:numberingOfSerials;_t
xc;rdvocab:performerNarratorAndOrPresenter;_t
xc;rdvocab:placeAndDateOfCapture;_t
xc;rdvocab:placeOfProduction;_t
xc;rdvocab:plateNumber;_t
xc;rdvocab:publisherNumber;_t
xc;rdvocab:scale;_t
xc;rdvocab:soundCharacteristics;_t
xc;rdvocab:statementOfResponsibilityRelatingToTitle;_t
xc;rdvocab:workTitle;_t
xc;xc:HoldingsExemplified;_t
xc;xc:ISO639-3LanguageName;_t
xc;xc:RestrictionsOnUse;_t
xc;xc:callNumber;_t
xc;xc:chronology;_t
xc;xc:contributor;_t
xc;xc:copyNumber;_t
xc;xc:coverage;_t
xc;xc:creator;_t
xc;xc:enumeration;_t
xc;xc:expressionManifested;_t
xc;xc:expressionOfWork;_t
xc;xc:expressionTitle;_t
xc;xc:holdingsOfManifestation;_t
xc;xc:identifier;_t
xc;xc:isPartOf;_t
xc;xc:itemOfHoldings;_t
xc;xc:location;_t
xc;xc:manifestationHeld;_t
xc;xc:manifestationOfExpression;_t
xc;xc:marksInscriptions;_t
xc;xc:otherPhysicalDetails;_t
xc;xc:pieceDesignation;_t
xc;xc:recordID;_t
xc;xc:relation;_t
xc;xc:serviceProvider;_t
xc;xc:spatial;_t
xc;xc:status;_t
xc;xc:subject;_t
xc;xc:tempLocation;_t
xc;xc:temporal;_t
xc;xc:textualHoldings;_t
xc;xc:thesisAdvisor;_t
xc;xc:type;_t
xc;xc:type007;_t
xc;xc:typeLeader06;_t
xc;xc:typeSMD;_t
xc;xc:workExpressed;_t
xc;dcterms:date;_d
xc;dcterms:dateAccepted;_d
xc;dcterms:dateCopyrighted;_d
xc;dcterms:dateSubmitted;_d
xc;dcterms:type_xxxxxx;_t
xc;dcterms:language_xxxxxx;_t
xc;rdarole:director_agentID;_i
xc;rdarole:editor_agentID;_i
xc;rdarole:illustrator_agentID;_i
xc;rdarole:performer_agentID;_i
xc;rdarole:producer_agentID;_i
xc;rdarole:translator_agentID;_i
xc;rdvocab:identifierForTheExpression_xxxxxx;_t
xc;xc:contributor_agentID;_i
xc;xc:manifestationOfExpression_xxxxxx;_t
xc;xc:workExpressed_xxxxxx;_t
xc;xc:expressionManifested_xxxxxx;_t
xc;xc:holdingsOfManifestation_xxxxxx;_t
xc;xc:identifier_xxxxxx;_t
xc;xc:isPartOf_workID;_i
xc;xc:recordID_xxxxxx;_t
xc;xc:itemOfHoldings_xxxxxx;_t
xc;xc:manifestationHeld_xxxxxx;_t
xc;xc:HoldingsExemplified_xxxxxx;_t
xc;xc:serviceProvider_xxxxxx;_t
xc;dcterms:abstract;_t
xc;dcterms:coverage_xxxxxx;_t
xc;dcterms:isReplacedBy_dcterms:ISSN;_t
xc;dcterms:replaces_dcterms:ISSN;_t
xc;dcterms:spatial_xxxxxx;_t
xc;dcterms:subject_xxxxxx;_t
xc;rdarole:artist_agentID;_t
xc;rdarole:author_agentID;_t
xc;rdarole:compiler_agentID;_t
xc;rdarole:composer_agentID;_t
xc;rdarole:speaker_agentID;_t
xc;rdvocab:identifierForTheWork_xxxxxx;_t
xc;xc:coverage_xxxxxx;_t
xc;xc:coverage_geoID;_t
xc;xc:creator_agentID;_t
xc;xc:expressionOfWork_xxxxxx;_t
xc;xc:relation_workID;_t
xc;xc:spatial_xxxxxx;_t
xc;xc:spatial_geoID;_t
xc;xc:subject_xxxxxx;_t
xc;xc:subject_subjID;_t
xc;xc:temporal_xxxxxx;_t
xc;xc:temporal_chronID;_t
xc;xc:type_xxxxxx;_t
xc;xc:type_subjID;_t
END;

  return $defaults;
}

/**
 * Get the list of field-type mappings in the form of an array. 
 * 
 * Each element of the array
 * has an 'id' and a 'type' element. The keys are the field names.
 * 
 */
function xc_search_field_type_get_all_by_schema($metadata_schema) {
  $sql = "SELECT * FROM {xc_search_field_type} WHERE metadata_schema = '%s'";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_field_type}', 
    'field_type_id'), $metadata_schema);
  $ret_val = array();
  while ($data = db_fetch_object($result)) {
    $ret_val[$data->field][] = $data->type;
  }
  return $ret_val;
  
}

