<?php
// $Id: xc_search_display_template.php,v 1.1.2.6 2009/12/16 15:00:37 mtwesley Exp $

function xc_search_display_template_get($entity_type, $display_type) {
  static $template_cache;
  if (!isset($template_cache)) {
    $template_cache = array();
  }
  
  if (!isset($template_cache[$entity_type]) || !isset($template_cache[$entity_type][$display_type])) {
    $sql = "SELECT template FROM {xc_search_display_template} 
      WHERE entity_type = '%s'
        AND display_type = '%s'";
    $result = db_query(db_rewrite_sql($sql, '{xc_search_display_template_element}',
      'element_id'), $entity_type, $display_type);
    $data = db_fetch_object($result);
    $template_cache[$entity_type][$display_type] = $data->template;
  }
  return $template_cache[$entity_type][$display_type];
}

function xc_search_display_template_load($xc_search_display_template) {
  $sql = "SELECT * FROM {xc_search_display_template} WHERE template_id = %d";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_display_template_element}',
      'element_id'), $xc_search_display_template);
  $data = db_fetch_object($result);
  return $data;
} 

function xc_search_display_template_title($record) {
  return t('Display template');
}

function xc_search_display_template_fieldset() {
  return array(
    'metadata_schema' => t('metadata schema'), 
    'entity_type'     => t('entity type'),
    'display_type'    => t('display type'), 
    'template'        => t('the displayed template'),
    'highlighter_id'  => t('the connected highlighter'),
  );
}

function xc_search_display_template_options() {
  $options = array('metadata_schema' => array());
  
  // metadata schema
  $options['metadata_schema'] = xc_format_get_schemas();
  
  // entities
  $options['entity_type'] = xc_format_get_entities();
  
  // display types
  $options['display_type'] = array(
    'title' => t('result\'s title'),
    'snippet' => t('result\'s body')
  );
  
  // highlighters
  $sql = "SELECT highlighter_id, name FROM {xc_search_highlighter}";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_highlighter}', 'higlighter_id'));
  $options['highlighter_id'][''] = '-- select optionally --';
  while ($data = db_fetch_object($result)) {
    $options['highlighter_id'][$data->highlighter_id] = $data->name;
  }
  
  
  return $options;
}

function xc_search_display_template_view($record) {
  return xc_util_view($record, 
    xc_search_display_template_fieldset(),
    xc_search_display_template_options(),
    array('filters' => array('template' => array('htmlentities', 'nl2br')))
  );
}

function xc_search_display_template_list() {
  $headers = array(t('Schema'), t('Entity'), t('Display type'), t('Template'));
  $rows = array();
  $options = xc_search_display_template_options();
  $sql = "SELECT * FROM {xc_search_display_template}";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_display_template_element}',
      'element_id'));
  while ($data = db_fetch_object($result)) {
    $rows[] = array(
      $data->metadata_schema,
      $data->entity_type,
      l($options['display_type'][$data->display_type], 'xc/search/displaying/display_template/'.$data->template_id),
      nl2br(htmlentities($data->template)) 
    );
  }
  return theme('table', $headers, $rows);
}

function xc_search_display_template_add() {
  drupal_set_title('Create a new template');
  return drupal_get_form('xc_search_display_template_form');
}

function xc_search_display_template_form() {
  $schema = drupal_get_schema_unprocessed('xc_search', 'xc_search_display_template');
  drupal_set_title($schema['description']);
  $options = array(
    'omit' => array('template_id'),
    'hidden' => array(),
    'label' => xc_search_display_template_fieldset(),
    'select' => xc_search_display_template_options(),
  // TODO: metadata_schema!!!
  );
  
  $form = xc_util_build_autoform($schema, $options);
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

function xc_search_display_template_edit_form($form_state, $record) {
  $form = xc_search_display_template_form($form_state, $record);
  $form['template_id'] = array(
    '#type'  => 'hidden',
    '#value' => $record->template_id,
  );
  
  foreach (xc_search_display_template_fieldset() as $name => $label) {
    $form[$name]['#default_value'] = $record->$name;
  }
    
  $form['submit']['#value'] = t('Save');
  $form['delete'] = array(
    '#type'  => 'submit',
    '#value' => t('Delete'),
  );

  return $form;
}

function xc_search_display_template_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $record = new stdClass();
  foreach (xc_search_display_template_fieldset() as $name => $label) {
    $record->$name = $values[$name];
  }

  $ret_val = drupal_write_record('xc_search_display_template', $record);
  if ($ret_val == SAVED_NEW) {
    drupal_set_message(t('Successfully added "%name" display_template!',
                         array('%name' => $record->name)));
    $form_state['redirect'] = 'xc/search/displaying/display_template/' . $record->template_id . '/view';
  }
  else {
    drupal_set_message(t('Unexpected error. Failed to create new display template.'));
  }
}

function xc_search_display_template_edit_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $record = new stdClass();
  $record->template_id             = $values['template_id'];
  foreach (xc_search_display_template_fieldset() as $name => $label) {
    $record->$name = $values[$name];
  }
    
  if ($form_state['clicked_button']['#value'] == t('Delete')) {
    $sql = "DELETE FROM {xc_search_display_template} WHERE template_id = %d";
    $result = db_query(db_rewrite_sql($sql, '{xc_search_display_template_element}',
      'element_id'), $record->template_id);

    if ($result == SAVED_DELETED) { // repository is deleted
      drupal_set_message(t('%name removed', array('%name' => $record->template_id)));
      $form_state['redirect'] = 'xc/search/displaying/display_template/list';
    }
    else {
      drupal_set_message(t('Unexpected error. Failed to remove display template.'));
    }
  }
  else {
    $ret_val = drupal_write_record('xc_search_display_template', $record, 'template_id');
    if ($ret_val == SAVED_UPDATED) {
      drupal_set_message(t('Successfully updated "%name" display template!',
                         array('%name' => $record->template_id)));
      $form_state['redirect'] = 'xc/search/displaying/display_template/' . $record->template_id . '/view';
    }
    else {
      drupal_set_message(t('Unexpected error. Failed modify display template.'));
    }
  }
}

